{
  Simple/Single Form
  2003-04 -
}

unit SForm;

interface

uses Windows, Messages;

type
  TNotifyEvent = procedure (Sender: TObject) of object;

  TWinControl = class(TObject)
    Handle: HWND;
    function GetClientRect: TRect;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); virtual;
    procedure SetSize(AWidth, AHeight: Integer); virtual;
    property ClientRect: TRect read GetClientRect;
    procedure Show; virtual;
    procedure Hide; virtual;
  end;

  TCommonControl = class(TWinControl)
    OrgWndProc: TFNWndProc;
    procedure CreateWnd(Parent: HWND; ALeft, ATop, AWidth, AHeight: Integer); virtual;
    procedure WndProc(var Message: TMessage); virtual;
  end;

  TSForm = class(TWinControl)
  protected
    FRegisteredClass: Boolean;
    FName: string;
    FWndClassName: string;
    FStayOnTop: Boolean;
    procedure SetStayOnTop(Value: Boolean);
  public
    Style: DWORD;
    ExStyle: DWORD;
    Brush: HBRUSH;
    Icon: HICON;
    Cursor: HCURSOR;
    ClassStyle: UINT;
    property Name: string read FName;
    property WndClassName: string read FWndClassName;
    property StayOnTop: Boolean read FStayOnTop write SetStayOnTop;
    constructor Create(const AName: string);
    procedure CreateWnd(ALeft, ATop, AWidth, AHeight: Integer);
    procedure Show; override;
    procedure Close;
    procedure WndProc(var Message: TMessage); virtual;
  end;

procedure MessageLoop(var nExitCode: Integer);
procedure MessageLoopWithAccel(hWnd: HWND; hAccel: HACCEL; var nExitCode: Integer);

implementation

const
  PropName = 'OBJECT';

{ TWinControl }

function TWinControl.GetClientRect: TRect;
begin
  Windows.GetClientRect(Handle, Result);
end;

procedure TWinControl.Setbounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  MoveWindow(Handle, ALeft, ATop, AWidth, AHeight, True);
end;

procedure TWinControl.SetSize(AWidth, AHeight: Integer);
begin
  SetWindowPos(Handle, 0, 0, 0, AWidth, AHeight,
    SWP_NOZORDER or SWP_NOMOVE or SWP_NOACTIVATE);
end;

procedure TWinControl.Show;
begin
  ShowWindow(Handle, SW_SHOW);
end;

procedure TWinControl.Hide;
begin
  ShowWindow(Handle, SW_HIDE);
end;

{ TCommonControl }

function ComCtlWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  CommonControl: TCommonControl;
  Message: TMessage;
begin
  CommonControl := TCommonControl(GetProp(hWnd, PropName));
  if Assigned(CommonControl) then
  begin
    Message.Msg := Msg;
    Message.WParam := wParam;
    Message.LParam := lParam;
    Message.Result := 0;
    CommonControl.WndProc(Message);
    Result := Message.Result;
  end else
    Result := 0;
end;

procedure TCommonControl.WndProc(var Message: TMessage);
begin
  with Message do
  begin
    case Msg of
      WM_NCDESTROY:
        begin
          RemoveProp(Handle, PropName);
          SetWindowLong(Handle, GWL_WNDPROC, Integer(@OrgWndProc));
          Result := 0;
        end;
    else
      Result := CallWindowProc(OrgWndProc, Handle, Msg, wParam, lParam);
    end;
  end;
end;

procedure TCommonControl.CreateWnd(Parent: HWND; ALeft, ATop, AWidth, AHeight: Integer);
begin
  SetProp(Handle, PropName, Integer(Self));
  OrgWndProc := TFnWndProc(SetWindowLong(Handle, GWL_WNDPROC, Integer(@ComCtlWindowProc)));
end;

{ TSForm }

function SFormWindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  Form: TSForm;
  Message: TMessage;
begin
  case Msg of
    WM_NCCREATE:
      begin
        SetProp(hWnd, PropName, Integer(PCreateStruct(lParam).lpCreateParams));
        TSForm(PCreateStruct(lParam).lpCreateParams).Handle := hWnd;
        Result := DefWindowProc(hWnd, Msg, wParam, lParam);
      end;
    WM_NCDESTROY:
      begin
        RemoveProp(hWnd, PropName);
        Result := DefWindowProc(hWnd, Msg, wParam, lParam);
      end;
  else
    Form := TSForm(GetProp(hWnd, PropName));
    if Assigned(Form) then
    begin
      Message.Msg := Msg;
      Message.WParam := wParam;
      Message.LParam := lParam;
      Message.Result := 0;
      Form.WndProc(Message);
      Result := Message.Result;
    end else
      Result := 0;
  end;
end;

procedure TSForm.WndProc(var Message: TMessage);
begin
  with Message do
    Result := DefWindowProc(Handle, Msg, WParam, LParam);
end;

constructor TSForm.Create(const AName: string);
begin
  FName := AName;
  FWndClassName := 'T' + FName + 'Class';
  FRegisteredClass := False;
  Brush := COLOR_3DFACE+1;
  Icon := LoadIcon(HInstance, 'MAINICON');
  Cursor := LoadCursor(0, IDC_ARROW);
  Style := WS_OVERLAPPEDWINDOW;
  ClassStyle := CS_DBLCLKS;
end;

procedure TSForm.SetStayOnTop(Value: Boolean);
begin
  FStayOnTop := Value;
  if Handle = 0 then
    Exit;
  if FStayOnTop then
    SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE)
  else
    SetWindowPos(Handle, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE);
end;

procedure TSForm.CreateWnd(ALeft, ATop, AWidth, AHeight: Integer);
var
  wc: TWndClass;
begin
  if not FRegisteredClass then
  begin
    FillChar(wc, SizeOf(wc), 0);
    wc.style         := ClassStyle;
    wc.lpfnWndProc   := @SFormWindowProc;
    wc.hInstance     := HInstance;
    wc.hIcon         := Icon;
    wc.hCursor       := Cursor;
    wc.hbrBackGround := Brush;
    wc.lpszClassName := PChar(FWndClassName);
    if RegisterClass(wc) = 0 then
      Exit;
    FRegisteredClass := True;
  end;

  Handle := 0;
  Handle := CreateWindowEx(ExStyle, PChar(FWndClassName), PChar(Name), Style,
    ALeft, ATop, AWidth, AHeight, 0, 0, HInstance, Self);
end;

procedure TSForm.Show;
begin
  ShowWindow(Handle, SW_SHOW);
  UpdateWindow(Handle);
end;

procedure TSForm.Close;
begin
  SendMessage(Handle, WM_SYSCOMMAND, SC_CLOSE, 0);
end;

procedure MessageLoop(var nExitCode: Integer);
var
  Msg: TMsg;
begin
  while Integer(GetMessage(Msg, 0, 0, 0)) <> 0 do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
  nExitCode := Msg.wParam;
end;

procedure MessageLoopWithAccel(hWnd: HWND; hAccel: HACCEL; var nExitCode: Integer);
var
  Msg: TMsg;
begin
  while Integer(GetMessage(Msg, 0, 0, 0)) <> 0 do
  begin
    if not Boolean(TranslateAccelerator(hWnd, hAccel, Msg)) then
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  end;
end;

end.
