unit VideoPlayer;

interface

uses
  Windows, ActiveX, MMSystem, DShow,
  RiffWalk;

type
  TVPModes = (vpPlay, vpPause, vpStop);

  TVideoPlayer = class(TObject)
  protected
    Figb: IGraphBuilder;
    Fimc: IMediaControl;
    Fims: IMediaSeeking;
    Fibv: IBasicVideo;
    //Fivf: IVideoFrameStep;
    Fiba: IBasicAudio;
    Fivw: IVideoWindow;
    TimerID: Integer;
    Duration: Comp;
    function GetVolume: Integer;
    procedure SetVolume(Value: Integer);
    function GetPosition: Integer;
    procedure SetPosition(Value: Integer);
  public
    Mode: TVpModes;
    FileName: string;
    Width: Integer;
    Height: Integer;
    Length: Integer;   // msec
    Loaded: Boolean;
    DisplayWnd: HWND;
    Fps: Double;
    AVIMainHeader: TAVIMainHeader;
    BitmapInfoHeader: TBitmapInfoHeader;
    WaveFormat: TWaveFormat;
    OnTimer: procedure of object;
    property Volume: Integer read GetVolume write SetVolume;
    property Position: Integer read GetPosition write SetPosition; // msec
    procedure FreeInterface;
    procedure LoadFile(const FileName: string);
    procedure Play;
    procedure Stop;
    procedure Pause;
    procedure PauseOnly;
    procedure SetWindowPosition(ALeft, ATop, AWidth, AHeight: Longint);
  end;

implementation

procedure TimeProc(IDEvent, uReserved, dwUser, dwReserved1, dwReserved2: Integer); stdcall;
begin
  with TVideoPlayer(dwUser) do
  begin
    if Loaded and Assigned(OnTimer) then
      OnTimer;
  end;
end;

function TVideoPlayer.GetVolume: Integer;
begin
  if Loaded then
    Fiba.get_Volume(Result)
  else
    Result := 0;
end;

procedure TVideoPlayer.SetVolume(Value: Integer);
begin
  if Loaded then Fiba.put_Volume(Value);
end;

function TVideoPlayer.GetPosition: Integer;
var
  cp: Comp;
begin
  if Loaded then
  begin
    Fims.GetCurrentPosition(cp);
    if cp >= Duration then
    begin
      Fimc.Stop; // ̂ȁ[
      Mode := vpStop;
    end;
    Result := Trunc(cp / 10000);
  end else
    Result := 0;
end;

procedure TVideoPlayer.SetPosition(Value: Integer);
var
  CurPos, StopPos: Comp;
begin
  if Loaded then
  begin
    CurPos := Value * 10000.0;
    if CurPos >= Duration then CurPos := Duration;
    StopPos := 0;
    Fims.SetPositions(CurPos, AM_SEEKING_AbsolutePositioning,
      StopPos, AM_SEEKING_NoPositioning);
    GetPosition;
  end;
end;

procedure TVideoPlayer.FreeInterface;
begin
  if Assigned(Fimc) then
    Fimc.Stop;
  if Assigned(Fivw) then
  begin
    Fivw.put_Visible(False);
    Fivw.put_Owner(OAHWND(0));
  end;
  Fimc := nil;
  Fivw := nil;
  Fims := nil;
  Figb := nil;
  Fibv := nil;
  //Fivf := nil;
  Fiba := nil;
  Loaded := False;
end;

procedure TVideoPlayer.Play;
var
  cp: Comp;
begin
  if Loaded then
  begin
    Fims.GetCurrentPosition(cp);
    if (cp = Duration) and (Mode = vpStop) then
      Position := 0;
    Fimc.Run;
    Mode := vpPlay;
    if TimerID <> 0 then
    begin
      timeKillEvent(TimerID);
      TimerID := 0;
    end;
    TimerID := timeSetEvent(100, 50, TimeProc, DWORD(Self), TIME_PERIODIC);
  end;
end;

procedure TVideoPlayer.Stop;
begin
  if Loaded then
  begin
    Fimc.Stop;
    Position := 0;
    Fimc.StopWhenReady;
    Mode := vpStop;
  end;
end;

procedure TVideoPlayer.PauseOnly;
begin
  if Loaded then
  begin
    timeKillEvent(TimerID);
    TimerID := 0;
    Fimc.Pause;
    Mode := vpPause;
  end;
end;

procedure TVideoPlayer.Pause;
begin
  if Mode = vpPause then
    Play
  else
    PauseOnly;
end;

procedure TVideoPlayer.LoadFile(const FileName: string);
var
  tpf: Double;
  F: File;
  ws: WideString;
  size: Integer;
begin
  Loaded := False;
  FillChar(AVIMainHeader, SizeOf(AVIMainHeader), 0);
  FillChar(BitmapInfoHeader, SizeOf(BitmapInfoHeader), 0);
  FillChar(WaveFormat, SizeOf(WaveFormat), 0);
  if Assigned(Fimc) then
  begin
    Fimc.Stop;
    FreeInterface;
  end;
  FileMode := 0; 
  AssignFile(F, FileName);
  try
    Reset(F, 1);
  except
    Exit;
  end;

  try
    if SearchChunk(F, 'RIFF', size) then
    begin
      SearchChunk(F, 'avih', size);
      BlockRead(F, AVIMainHeader, SizeOf(AVIMainHeader));
      SearchChunk(F, 'strf', size);
      Seek(F, FilePos(F)+8);
      BlockRead(F, BitmapInfoHeader, SizeOf(BitmapInfoHeader));
      SearchChunk(F, 'strf', size);
      Seek(F, FilePos(F)+8);
      BlockRead(F, WaveFormat, SizeOf(WaveFormat));
    end;
  except
  end;
  CloseFile(F);

  try
    CoCreateInstance(CLSID_FilterGraph, nil, CLSCTX_INPROC_SERVER,
      IID_IGraphBuilder, Figb);
    ws := FileName;
    if Figb.RenderFile(PWideChar(ws), nil) <> S_OK then
    begin
      FreeInterface;
      Exit;
    end;
    Fivw:= Figb as IVideoWindow;
    Fivw.put_Owner(OAHWND(DisplayWnd));
    Fivw.put_WindowStyle(WS_CHILD or WS_CLIPSIBLINGS);
    Fivw.put_MessageDrain(DisplayWnd);
    Fimc:= Figb as IMediaControl;
    Fims:= Figb as IMediaSeeking;
    //Fims.SetTimeFormat(TIME_FORMAT_FRAME);
    Fims.SetTimeFormat(TIME_FORMAT_MEDIA_TIME);
    Fibv := Figb as IBasicVideo;
    Fibv.GetVideoSize(Width, Height);
    Fibv.get_AvgTimePerFrame(tpf);
    if tpf = 0 then
    begin
      if AVIMainHeader.dwMicroSecPerFrame > 0 then
        Fps := 1000000 / AVIMainHeader.dwMicroSecPerFrame
      else
        Fps := 0;
    end else
      Fps := 1 / tpf;
    Fims.GetDuration(Duration);
    Length := Trunc(Duration / 10000);
    Self.FileName := FileName;
    Position := 0;
    Loaded := True;
    //Fivf := Figb as IVideoFrameStep;
    //Fims.SetRate(2.0);
    Fiba := Figb as IBasicAudio;
  except
    FreeInterface;
  end;
end;

procedure TVideoPlayer.SetWindowPosition(ALeft, ATop, AWidth, AHeight: Longint);
begin
  Fivw.SetWindowPosition(ALeft, ATop, AWidth, AHeight);
end;

end.
